# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'menu.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainMenu(object):
    def setupUi(self, MainMenu):
        MainMenu.setObjectName("MainMenu")
        MainMenu.resize(816, 605)
        MainMenu.setStyleSheet("color: #1d1e1f;")
        self.gridLayout_2 = QtWidgets.QGridLayout(MainMenu)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setContentsMargins(0, 7, -1, -1)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(5)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.line_4 = QtWidgets.QFrame(MainMenu)
        self.line_4.setStyleSheet("color: #b187eb;")
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setLineWidth(2)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_2.addWidget(self.line_4)
        self.line_5 = QtWidgets.QFrame(MainMenu)
        self.line_5.setStyleSheet("color: #87CEEB;")
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setLineWidth(2)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_2.addWidget(self.line_5)
        self.line_6 = QtWidgets.QFrame(MainMenu)
        self.line_6.setStyleSheet("color: #00ffff;")
        self.line_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_6.setLineWidth(2)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_2.addWidget(self.line_6)
        self.verticalLayout.addLayout(self.verticalLayout_2)
        spacerItem = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(100, -1, -1, -1)
        self.verticalLayout_3.setSpacing(40)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.Add = QtWidgets.QPushButton(MainMenu)
        self.Add.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Add.sizePolicy().hasHeightForWidth())
        self.Add.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.Add.setFont(font)
        self.Add.setAutoFillBackground(False)
        self.Add.setStyleSheet("border: bold")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(
            ":/img/kisspng-computer-icons-download-button-symbol-plus-5abd9e3ed95b29.3600526615223762548903-removebg-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Add.setIcon(icon)
        self.Add.setIconSize(QtCore.QSize(50, 50))
        self.Add.setAutoDefault(True)
        self.Add.setDefault(True)
        self.Add.setObjectName("Add")
        self.verticalLayout_3.addWidget(self.Add, 0, QtCore.Qt.AlignLeft)
        self.Run = QtWidgets.QPushButton(MainMenu)
        self.Run.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Run.sizePolicy().hasHeightForWidth())
        self.Run.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.Run.setFont(font)
        self.Run.setStyleSheet("border: bold;\n"
                               "")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(
            ":/img/run-removebg-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Run.setIcon(icon1)
        self.Run.setIconSize(QtCore.QSize(50, 50))
        self.Run.setAutoDefault(True)
        self.Run.setDefault(True)
        self.Run.setObjectName("Run")
        self.verticalLayout_3.addWidget(self.Run, 0, QtCore.Qt.AlignLeft)
        self.Settings = QtWidgets.QPushButton(MainMenu)
        self.Settings.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Settings.sizePolicy().hasHeightForWidth())
        self.Settings.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.Settings.setFont(font)
        self.Settings.setStyleSheet("border: bold")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(
            ":/img/download__3_-removebg-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Settings.setIcon(icon2)
        self.Settings.setIconSize(QtCore.QSize(50, 50))
        self.Settings.setObjectName("Settings")
        self.verticalLayout_3.addWidget(self.Settings, 0, QtCore.Qt.AlignLeft)
        self.verticalLayout.addLayout(self.verticalLayout_3)
        spacerItem1 = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi(MainMenu)
        QtCore.QMetaObject.connectSlotsByName(MainMenu)

    def retranslateUi(self, MainMenu):
        _translate = QtCore.QCoreApplication.translate
        MainMenu.setWindowTitle(_translate("MainMenu", "Form"))
        self.Add.setText(_translate("MainMenu", "  Add Files"))
        self.Run.setText(_translate("MainMenu", "  Run Analysis"))
        self.Settings.setText(_translate("MainMenu", "  Settings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainMenu = QtWidgets.QWidget()
    ui = Ui_MainMenu()
    ui.setupUi(MainMenu)
    MainMenu.show()
    sys.exit(app.exec_())



import identifyPeople
import cv2 as cv
from identifyPeople import crop_subjects_from_segmented_image as identify
from matplotlib import pyplot as plt
import os
import shutil
from random import randint


def make_set():
    unfound = 0

    print("Starting... ")

    new_set = 'bestOf/resources/identifiedFaces'
    directory = 'bestOf/resources/lfw'

    if os.path.isdir(new_set):
        shutil.rmtree(new_set)

    os.mkdir(new_set)

    for folder in os.listdir(directory):
        os.mkdir(new_set + '/' + folder + '_Identified')
        for img in os.listdir(os.path.join(directory, folder)):

            pik = cv.imread(os.path.join(directory, folder, img))
            image = cv.cvtColor(pik, cv.COLOR_BGR2RGB)

            faces, bounds = identify(image)
            if len(faces) == 0:
                unfound += 1

            for i, face in enumerate(faces):
                strings = img.split('.')
                name = ""
                if i > 0:
                    name = strings[0] + '_Identified' + \
                        str(i) + '.' + strings[1]
                else:
                    name = strings[0] + '_Identified' + '.' + strings[1]

                cv.imwrite(os.path.join(
                    new_set, folder + '_Identified', name), face)

    print('Number of failed identifications: ', str(unfound))


def crop_all_images():
    counter = 0
    counter_cropped = 0
    print('Starting to crop all images...')
    new_set = 'bestOf/resources/identifiedFacesCropped'
    directory = 'bestOf/resources/identifiedFaces'

    if os.path.isdir(new_set):
        shutil.rmtree(new_set)

    os.mkdir(new_set)

    for folder in os.listdir(directory):
        os.mkdir(new_set + '/' + folder + '_Cropped')
        print("Working on: " + folder)
        for i, img in enumerate(os.listdir(os.path.join(directory, folder))):
            counter += 1
            strings = img.split('.')
            name = strings[0] + str(i) + '.' + strings[1]

            image = cv.imread(os.path.join(directory, folder, img))
            rand_x = randint(20, 25)
            rand_y = randint(20, 25)
            cropped_list = crop_an_image(image, rand_x / 100, rand_y / 100)

            index = randint(0, 3)
            cv.imwrite(os.path.join(new_set, folder +
                                    '_Cropped', name), cropped_list[index])
            counter_cropped += 1

        print("----Finished----\n")

    print("---------------------Done With Everything---------------------")
    print("Out of " + str(counter) + " images\n" +
          str(counter_cropped) + " cropped images were created")


def crop_an_image(image, x, y):
    height = int(image.shape[0] * y)
    length = int(image.shape[1] * x)
    b_height = (image.shape[0] - height)
    b_length = (image.shape[1] - length)

    left_cutoff = image[:, length:]
    top_cutoff = image[height:, :]
    right_cutoff = image[:, :b_length]
    bottom_cutoff = image[:b_height, :]

    return [left_cutoff, top_cutoff, right_cutoff, bottom_cutoff]


# make_set()
if __name__ == '__main__':
    crop_all_images()



import torch
from torchvision import transforms
from torch.autograd import Variable
import cv2 as cv

from bestOf.backend.classDefinitions import BlinkAndCropNet
import os
import sys
sitePackageList = sys.path
sitePackPath = ""

for i in sitePackageList:
    if not os.path.isdir(i):
        continue
    if "bestOf" in os.listdir(i):
        sitePackPath = os.path.join(
            i, './bestOf/backend/saved_models/cropped_faces.pth')

sitePackPath = sitePackPath.replace("\\", "/")

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
model = BlinkAndCropNet()

# sitePackPath = './bestOf/backend/saved_models/cropped_faces.pth'


model.load_state_dict(torch.load(sitePackPath, map_location=device))
model.eval()

transform = transforms.Compose(
    [transforms.ToTensor()])


def test(image):
    try:
        image = cv.cvtColor(image, cv.COLOR_BGR2RGB)
        image = cv.resize(image, (80, 80), interpolation=cv.INTER_AREA)

        image_tensor = transform(image)
        image_tensor = image_tensor.unsqueeze(0)

        input_var = Variable(image_tensor)
        input_var = input_var.to(device)
        output = model(input_var)
        output = torch.round(output)

        return output.data.cpu().numpy()[0][0]

    except Exception as e:
        # print(e)
        return 0


# test(cv.imread('../resources/blinkset/blinkers/closed_eye_0059.jpg_face_2.jpg'))



import bestOf.backend.evaluateSharpness as evaluateSharpness
import bestOf.backend.evaluateCentering as evaluateCentering
import bestOf.backend.evaluateLighting as evaluateLighting
import bestOf.backend.evaluateResolution as evaluateResolution

import collections


def create_sharpness_map(images, image_info, groups, weight, progress_func, progress, max_progress):
    sharpness_scores = []

    for idx, image in enumerate(images):
        sharpness_scores.append(evaluateSharpness.evaluate_sharpness(
            image))
        progress += 1
        progress_func("sharpness", image_info[idx][1], int(
            progress / max_progress * 100))
    # index to normalized sharpness score + half of normalized subject sharpness score
    sharpness_map = {}
    for group in groups:
        group_sharpness_scores = []
        for index in group:
            for pos, item in enumerate(image_info):
                if item[0] == index:
                    group_sharpness_scores.append(
                        sharpness_scores[pos])
                    break
            progress += 1
            progress_func("sharpness", image_info[pos][1], int(
                progress / max_progress * 100))
        group_sharpness_scores = evaluateSharpness.normalize_sharpness_scores(
            group_sharpness_scores)

        avg_subject_sharpness_scores = []
        for index in group:
            for pos, item in enumerate(image_info):
                if item[0] == index:
                    subject_sharpness_scores = item[3]
                    if not len(subject_sharpness_scores):
                        avg_subject_sharpness_scores.append(0)
                        continue

                    avg_subject_sharpness_scores.append(
                        sum(subject_sharpness_scores) / len(subject_sharpness_scores))
            progress += 1
            progress_func("sharpness", image_info[pos][1], int(
                progress / max_progress * 100))
        avg_subject_sharpness_scores = evaluateSharpness.normalize_sharpness_scores(
            avg_subject_sharpness_scores)

        for idx_in_group, index in enumerate(group):
            sharpness_map[index] = weight * ((3 / 4) * group_sharpness_scores[idx_in_group] + (
                avg_subject_sharpness_scores[idx_in_group] / 4))

    return sharpness_map, progress


def create_centering_map(image_info, groups, weight, progress_func, progress, max_progress):
    centering_map = {}

    for group in groups:
        group_centering_scores = []
        for index in group:
            for pos, item in enumerate(image_info):
                if item[0] == index:
                    group_centering_scores.append(image_info[pos][4])
                    break
            progress += 1
            progress_func("centering", image_info[pos][1], int(
                progress / max_progress * 100))
        # print(group_centering_scores)
        group_centering_scores = evaluateCentering.normalize_centering_scores(
            group_centering_scores)

        for idx_in_group, index in enumerate(group):
            centering_map[index] = weight * \
                group_centering_scores[idx_in_group]

    return centering_map, progress


def create_lighting_map(image_info, groups, weight, progress_func, progress, max_progress):
    lighting_map = {}
    for group in groups:
        avg_subject_lighting_scores = []
        for index in group:
            for pos, item in enumerate(image_info):
                if item[0] == index:
                    subject_lighting_scores = item[5]
                    if not len(subject_lighting_scores):
                        avg_subject_lighting_scores.append(0)
                        continue

                    avg_subject_lighting_scores.append(
                        sum(subject_lighting_scores) / len(subject_lighting_scores))
            progress += 1
            progress_func("lighting", image_info[pos][1], int(
                progress / max_progress * 100))
        avg_subject_lighting_scores = evaluateLighting.normalize_lighting_scores(
            avg_subject_lighting_scores)

        for idx_in_group, index in enumerate(group):
            lighting_map[index] = weight * \
                avg_subject_lighting_scores[idx_in_group]

    return lighting_map, progress


def create_resolution_map(images, image_info, groups, weight, progress_func, progress, max_progress):
    resolution_scores = []
    resolution_map = {}

    for idx, image in enumerate(images):
        resolution_scores.append(evaluateResolution.evaluate_resolution(
            image))
        progress += 1
        progress_func("resolution", image_info[idx][1], int(
            progress / max_progress * 100))

    for group in groups:
        group_resolution_scores = []
        for index in group:
            for pos, item in enumerate(image_info):
                if item[0] == index:
                    group_resolution_scores.append(
                        resolution_scores[pos])
                    break
            progress += 1
            progress_func("resolution", image_info[pos][1], int(
                progress / max_progress * 100))
        group_resolution_scores = evaluateResolution.normalize_resolution_scores(
            group_resolution_scores)

        for idx_in_group, index in enumerate(group):
            resolution_map[index] = weight * \
                group_resolution_scores[idx_in_group]
    return resolution_map, progress


def create_total_score_map(image_info, sharpness_map, centering_map, lighting_map, resolution_map):
    overall_score_map = collections.Counter(sharpness_map) + collections.Counter(
        centering_map) + collections.Counter(lighting_map) + collections.Counter(resolution_map)

    overall_score_map = dict(overall_score_map)

    if not len(overall_score_map.keys()):
        for item in image_info:
            overall_score_map[item[0]] = item[2]
        return overall_score_map

    for item in image_info:
        overall_score_map[item[0]] += item[2]

    return overall_score_map



import sys
import os


def get_site_package_path(filename):
    sitePackageList = sys.path
    sitePackPath = ""

    for i in sitePackageList:
        if not os.path.isdir(i):
            continue
        if "bestOf" in os.listdir(i):
            sitePackPath = os.path.join(
                i, filename)

    sitePackPath = sitePackPath.replace("\\", "/")
    return sitePackPath




from PyQt5.QtGui import *
import sys
import numpy as np

from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget, QFileDialog
from torch._C import layout
import bestOf.frontend.res
import json
import collections
from bestOf.frontend.analyzing_page import Ui_AnalyzingPage
from bestOf.frontend.main import Ui_MainWindow
from bestOf.frontend.menu import Ui_MainMenu
from bestOf.frontend.settings import Ui_Settings
from bestOf.frontend.results import Ui_Results
from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5 import QtCore
from threading import Thread
import time
import shutil
import os
sys.path.insert(1, 'bestOf/backend')

import bestOf.backend.similarity as similarity
import bestOf.backend.blinkDetector as blinkDetector
import bestOf.backend.cropDetector as cropDetector
import bestOf.backend.evaluateSharpness as evaluateSharpness
import bestOf.backend.evaluateLighting as evaluateLighting
import bestOf.backend.identifyPeople as identifyPeople
import bestOf.backend.evaluateCentering as evaluateCentering
import bestOf.backend.createScoreMaps as createScoreMaps
import bestOf.backend.sitePackagePathConstructor as sitePackagePathConstructor


def read_settings():
    path = sitePackagePathConstructor.get_site_package_path(
        './bestOf/backend/settings.json')
    settings = {}
    with open(path, 'r') as settings_file:
        settings = json.load(settings_file)
    return settings


def write_settings(settings):
    path = sitePackagePathConstructor.get_site_package_path(
        './bestOf/backend/settings.json')
    with open(path, 'w') as settings_file:
        json.dump(settings, settings_file)


def loadImages(filenames):
    for filename in filenames:
        image = identifyPeople.read_img(filename)
        yield (image * 255).astype(np.uint8)[:, :, :3] if 'png' in filename else image


def simulateAnalyzing(filenames, imagelist, groups, settings, callback):
    imagelistLen = len(imagelist)

    maxProgress = 0
    maxProgress += min(1, settings["sharpness"]) * imagelistLen * 3
    maxProgress += min(1, settings["centering"]) * imagelistLen
    maxProgress += min(1, settings["lighting"]) * imagelistLen
    maxProgress += min(1, settings["resolution"]) * imagelistLen
    progress = 0

    sharpness_map = {}
    centering_map = {}
    lighting_map = {}
    resolution_map = {}

    if settings["sharpness"]:
        image_generator = loadImages(filenames)

        sharpness_map, progress = createScoreMaps.create_sharpness_map(
            image_generator, imagelist, groups, settings["sharpness"], callback, progress, maxProgress)
        # print(sharpness_map)

    if settings["centering"]:
        centering_map, progress = createScoreMaps.create_centering_map(
            imagelist, groups, settings["centering"], callback, progress, maxProgress)
        # print(centering_map)

    if settings["lighting"]:
        lighting_map, progress = createScoreMaps.create_lighting_map(
            imagelist, groups, settings["lighting"], callback, progress, maxProgress)
        # print(lighting_map)

    if settings["resolution"]:
        image_generator = loadImages(filenames)

        resolution_map, progress = createScoreMaps.create_resolution_map(
            image_generator, imagelist, groups, settings["resolution"], callback, progress, maxProgress)
        # print(resolution_map)

    total_score_map = createScoreMaps.create_total_score_map(
        imagelist, sharpness_map, centering_map, lighting_map, resolution_map)

    # print('\n\n')
    # print('Total Score Map', total_score_map)
    # print('\n\n')

    sorted_groups = []
    for group in groups:
        sorted_group = []
        for index in group:
            sorted_group.append((index, total_score_map[index]))
        sorted_group = sorted(sorted_group, key=lambda x: x[1], reverse=True)
        sorted_groups.append([element[0] for element in sorted_group])

    return sorted_groups


class BestOfApp(QObject):
    progressChangedSignal = pyqtSignal(int)
    statusChangedSignal = pyqtSignal(str, str)
    analyzingCriteriaChangedSignal = pyqtSignal(str)
    analyzingImageChangedSignal = pyqtSignal(str)
    analyzingFinishedSignal = pyqtSignal()

    def __init__(self):
        super().__init__()
        self.MainWindow = QMainWindow()
        self.Ui_MainWindow = Ui_MainWindow()
        self.Ui_MainWindow.setupUi(self.MainWindow)
        self.MainMenu = QWidget()
        self.Ui_MainMenu = Ui_MainMenu()
        self.Ui_MainMenu.setupUi(self.MainMenu)
        self.Settings = QWidget()
        self.Ui_Settings = Ui_Settings()
        self.Ui_Settings.setupUi(self.Settings)
        self.AnalyzingPage = QWidget()
        self.Ui_AnalyzingPage = Ui_AnalyzingPage()
        self.Ui_AnalyzingPage.setupUi(self.AnalyzingPage)
        self.Results = QWidget()
        self.Ui_Results = Ui_Results()
        self.Ui_Results.setupUi(self.Results)

        self.MainWindow.installEventFilter(self)

        self.Ui_MainWindow.stackedWidget.addWidget(self.MainMenu)
        self.Ui_MainWindow.stackedWidget.addWidget(self.Settings)
        self.Ui_MainWindow.stackedWidget.addWidget(self.AnalyzingPage)
        self.Ui_MainWindow.stackedWidget.addWidget(self.Results)

        self.Ui_MainWindow.progressBar.setVisible(False)

        self.Ui_Settings.criteriaChangedSignal.connect(self.onCriteriaChanged)
        self.Ui_Settings.thresholdChangedSignal.connect(
            self.onThresholdChanged)

        self.Ui_MainMenu.Add.clicked.connect(self.getFiles)
        self.Ui_MainMenu.Run.clicked.connect(self.analyzeImages)

        self.Ui_Results.imageToggledSignal.connect(self.onImageToggled)
        self.Ui_Results.downloadSelectedSignal.connect(self.downloadSelected)

        self.Ui_MainMenu.Settings.clicked.connect(
            lambda: self.Ui_MainWindow.stackedWidget.setCurrentIndex(1))
        self.Ui_Settings.Back.clicked.connect(
            lambda: self.Ui_MainWindow.stackedWidget.setCurrentIndex(0))
        self.Ui_Results.Back.clicked.connect(
            lambda: self.Ui_MainWindow.stackedWidget.setCurrentIndex(0))

        self.analyzingCriteriaChangedSignal.connect(
            self.Ui_AnalyzingPage.setCriteria)
        self.analyzingImageChangedSignal.connect(
            self.Ui_AnalyzingPage.setImage)
        self.progressChangedSignal.connect(self.Ui_MainWindow.changeProgress)
        self.statusChangedSignal.connect(self.Ui_MainWindow.changeStatus)
        self.analyzingFinishedSignal.connect(self.onAnalyzingFinished)

        self.settings = read_settings()
        self.Ui_Settings.setSettings(self.settings)

        self.filenames = []
        self.imageList = []
        self.groups = []
        self.downloadingList = []

        self.MainWindow.show()

    def onCriteriaChanged(self, criteria, value):
        self.settings[criteria] = value

    def onThresholdChanged(self, value):
        self.settings["threshold"] = value

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.Close:
            write_settings(self.settings)
        return super().eventFilter(obj, event)

    def getFiles(self):
        self.filenames = []
        self.imageList = []
        self.groups = []
        self.downloadingList = []

        file = QFileDialog.getOpenFileNames(
            self.MainWindow, 'Add Files', QtCore.QDir.currentPath(), "Image Files (*.png *.jpg)")
        if len(file[0]) == 0:
            self.Ui_MainWindow.changeStatus("No files selected!", "red")
            return
        self.Ui_MainWindow.changeStatus("Reading images...")
        thread = Thread(target=self.loadFiles, args=(file,))
        thread.start()

    def loadFiles(self, file):
        maxProgress = len(file[0])
        progress = 0

        loaded = []
        if len(self.imageList):
            loaded = [item[1] for item in self.imageList]

        image_generator = loadImages(list(file[0]) + loaded)
        self.filenames = list(file[0])
        vectors = []
        for image in image_generator:
            v = similarity.generate_feature_vector(image)
            vectors.append(v)
            progress += 1
            self.progressChangedSignal.emit(int(progress / maxProgress * 100))

            self.Ui_MainWindow.changeStatus("Running default analysis...")
            progress = 0
            maxProgress = len(file[0])
            self.progressChangedSignal.emit(int(progress / maxProgress * 100))

        groups = similarity.group(
            vectors, threshold=self.settings["threshold"])
        self.groups = groups

        # print(self.groups)

        image_generator = loadImages(list(file[0]))

        default_scores = []
        subject_sharpness_scores = []
        subject_lighting_scores = []
        centering_scores = []

        for image in image_generator:
            # print('Scanning Image...')
            subjects, bounds_list = identifyPeople.crop_subjects_from_segmented_image(
                image, n=4)

            # print("len of subjects", len(subjects))
            blinks = 0
            crops = 0

            subject_sharpness_scores.append([])
            subject_lighting_scores.append([])

            centering_scores.append(
                evaluateCentering.evaluate_centering(bounds_list, image))

            for sub in subjects:
                # print('Scanning Subject...')
                if blinkDetector.test(sub):
                    blinks += 1
                if cropDetector.test(sub):
                    crops += 1

                subject_sharpness_scores[-1].append(
                    evaluateSharpness.evaluate_sharpness(sub))

                subject_lighting_scores[-1].append(
                    evaluateLighting.evaluate_lighting(sub))

            if len(subjects) == 0:
                default_scores.append(0)
                progress += 1
                self.progressChangedSignal.emit(
                    int(progress / maxProgress * 100))
                continue

            blink_score = (len(subjects) - blinks) / len(subjects)
            crop_score = (len(subjects) - crops) / len(subjects)

            default_score = blink_score + crop_score
            default_scores.append(default_score)

            progress += 1
            self.progressChangedSignal.emit(int(progress / maxProgress * 100))

        if len(self.imageList):
            max_index = max([elem[0] for elem in self.imageList])
        else:
            max_index = -1

        final = zip(
            range(max_index + 1, max_index + 1 + len(file[0])), file[0], default_scores, subject_sharpness_scores, centering_scores, subject_lighting_scores)

        if len(self.imageList):
            final = list(final) + self.imageList
        # final = sorted(final, key=lambda x: x[2], reverse=True)
        # print(final)
        self.imageList = list(final)
        self.statusChangedSignal.emit(
            "Successfully loaded images! Check Settings, then click Run Analysis to process your images.", "green")

    def analyzeImages(self):
        if len(self.imageList) == 0:
            self.Ui_MainWindow.changeStatus(
                "Nothing to process! Upload images by clicking the Add Files button.", "red")
            return
        self.downloadingList = []
        self.Ui_MainWindow.changeStatus("Analyzing...")
        self.Ui_MainWindow.stackedWidget.setCurrentIndex(2)
        thread = Thread(target=self.analyzingThread)
        thread.start()

    def analyzingThread(self):
        self.groups = simulateAnalyzing(self.filenames, self.imageList, self.groups,
                                        self.settings, self.analyzingCallback)
        self.analyzingFinishedSignal.emit()

    def analyzingCallback(self, criteria, image, progress):
        self.analyzingCriteriaChangedSignal.emit(criteria)
        self.analyzingImageChangedSignal.emit(image)
        self.progressChangedSignal.emit(progress)

    def onAnalyzingFinished(self):
        self.Ui_MainWindow.changeStatus(
            "Analyzing finished successfully", "green")
        self.Ui_MainWindow.stackedWidget.setCurrentIndex(3)
        self.Ui_Results.display(self.imageList, self.groups)

    def onImageToggled(self, selected, id):
        if selected:
            self.downloadingList.append(id)
        else:
            self.downloadingList.remove(id)

    def downloadSelected(self):
        folder = QFileDialog.getExistingDirectory(
            self.MainWindow, "Select Directory")
        thread = Thread(target=self.downloadingThread, args=(folder,))
        thread.start()

    def downloadingThread(self, folder):
        maxProgress = len(self.downloadingList)
        progress = 0
        for elem in self.imageList:
            if elem[0] in self.downloadingList:
                shutil.copyfile(elem[1], os.path.join(
                    folder, os.path.basename(elem[1])))
                progress += 1
                self.progressChangedSignal.emit(
                    int(progress / maxProgress * 100))

        self.statusChangedSignal.emit("Downloading Finished", "green")


def main():
    import sys
    app = QApplication(sys.argv)
    bestOfApp = BestOfApp()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



import cv2 as cv
import mediapipe as mp
from matplotlib import pyplot as plt
import numpy as np
import bestOf.backend.similarity as similarity

FACE_DETECTOR = mp.solutions.face_detection.FaceDetection(
    model_selection=1, min_detection_confidence=0.7)


def array_is_subset(superarray, subarray):
    return set(np.unique(subarray)).issubset(set(np.unique(superarray)))


def read_img(filename):
    return plt.imread(filename)


def save_img(filename, img):
    cv.imwrite(filename, img)


def save_img_colored(filename, img):
    cv.imwrite(filename, cv.cvtColor(img, cv.COLOR_RGB2BGR))


def show_img(image):
    cv.imshow('Image', image)
    cv.waitKey(0)


def detect_faces(image):
    if image is None:
        raise Exception('No image found.')

    return FACE_DETECTOR.process(image).detections


def crop_subjects_from_segmented_image(image, n=8):
    im_height, im_width, _ = image.shape

    delta_h = int(im_height / n)
    delta_w = int(im_width / n)

    #print(delta_h, delta_w)

    all_subjects = []
    all_bounds = []

    for i in range(n):
        for j in range(n):
            seg = image[int(i * delta_h):int((i + 1) * delta_h),
                        int(j * delta_w):int((j + 1) * delta_w)]
            subs, bounds_list = crop_subjects(np.copy(seg, order='K'))

            for bounds in bounds_list:
                seg_width = int((j + 1) * delta_w) - int(j * delta_w)
                seg_height = int((i + 1) * delta_h) - int(i * delta_h)

                x_dist_to_seg = int(j * delta_w)
                x_dist_from_seg_to_face = int(bounds.xmin * seg_width)
                bounds.xmin = (x_dist_to_seg +
                               x_dist_from_seg_to_face) / im_width
                bounds.width = bounds.width * seg_width / im_width

                y_dist_to_seg = int(i * delta_h)
                y_dist_from_seg_to_face = int(bounds.ymin * seg_height)
                bounds.ymin = (y_dist_to_seg +
                               y_dist_from_seg_to_face) / im_height
                bounds.height = bounds.height * seg_height / im_height

            all_subjects.extend(subs)
            all_bounds.extend(bounds_list)

    intermediate_range = [
        element + 0.5 for element in range(n) if element != n - 1]

    # print(intermediate_range)

    for i in intermediate_range:
        for j in range(n):
            seg = image[int(i * delta_h):int((i + 1) * delta_h),
                        int(j * delta_w):int((j + 1) * delta_w)]
            subs, bounds_list = crop_subjects(np.copy(seg, order='K'))

            for bounds in bounds_list:
                seg_width = int((j + 1) * delta_w) - int(j * delta_w)
                seg_height = int((i + 1) * delta_h) - int(i * delta_h)

                x_dist_to_seg = int(j * delta_w)
                x_dist_from_seg_to_face = int(bounds.xmin * seg_width)
                bounds.xmin = (x_dist_to_seg +
                               x_dist_from_seg_to_face) / im_width
                bounds.width = bounds.width * seg_width / im_width

                y_dist_to_seg = int(i * delta_h)
                y_dist_from_seg_to_face = int(bounds.ymin * seg_height)
                bounds.ymin = (y_dist_to_seg +
                               y_dist_from_seg_to_face) / im_height
                bounds.height = bounds.height * seg_height / im_height

            for idx, sub in enumerate(subs):
                for existing_sub in all_subjects:
                    u = similarity.generate_feature_vector(sub)
                    v = similarity.generate_feature_vector(existing_sub)

                    if array_is_subset(existing_sub, sub[0:int(sub.shape[0] / 4), 0:int(sub.shape[1] / 4)]) or similarity.equiv(u, v, 0.8):
                        break
                else:
                    all_subjects.append(sub)
                    all_bounds.append(bounds_list[idx])

    for i in range(n):
        for j in intermediate_range:
            seg = image[int(i * delta_h):int((i + 1) * delta_h),
                        int(j * delta_w):int((j + 1) * delta_w)]
            subs, bounds_list = crop_subjects(np.copy(seg, order='K'))

            for bounds in bounds_list:
                seg_width = int((j + 1) * delta_w) - int(j * delta_w)
                seg_height = int((i + 1) * delta_h) - int(i * delta_h)

                x_dist_to_seg = int(j * delta_w)
                x_dist_from_seg_to_face = int(bounds.xmin * seg_width)
                bounds.xmin = (x_dist_to_seg +
                               x_dist_from_seg_to_face) / im_width
                bounds.width = bounds.width * seg_width / im_width

                y_dist_to_seg = int(i * delta_h)
                y_dist_from_seg_to_face = int(bounds.ymin * seg_height)
                bounds.ymin = (y_dist_to_seg +
                               y_dist_from_seg_to_face) / im_height
                bounds.height = bounds.height * seg_height / im_height

            for idx, sub in enumerate(subs):
                for existing_sub in all_subjects:
                    u = similarity.generate_feature_vector(sub)
                    v = similarity.generate_feature_vector(existing_sub)

                    if array_is_subset(existing_sub, sub[0:int(sub.shape[0] / 4), 0:int(sub.shape[1] / 4)]) or similarity.equiv(u, v, 0.8):
                        break
                else:
                    all_subjects.append(sub)
                    all_bounds.append(bounds_list[idx])

    if n == 1:
        return all_subjects, all_bounds
    else:
        larger_scale_subjects, larger_scale_bounds = crop_subjects_from_segmented_image(
            image, n=n - 1)
        if len(larger_scale_subjects) >= len(all_subjects):
            return larger_scale_subjects, larger_scale_bounds
        else:
            return all_subjects, all_bounds


def get_subject_bounds(face_info):
    return face_info.location_data.relative_bounding_box


# def correct_eye_bounds(top_left, bottom_right):
#     new_top_left = min(top_left[0], bottom_right[0]), min(
#         top_left[1], bottom_right[1])
#     new_bottom_right = max(top_left[0], bottom_right[0]), max(
#         top_left[1], bottom_right[1])
#     return new_top_left, new_bottom_right


def is_left_eye_higher(face_info):
    return face_info['keypoints']['left_eye'][1] < face_info['keypoints']['right_eye'][1]


def get_eye_bounds(face_info):
    # Average eye diameter is roughly 23 mm: Bekerman, Inessa & Gottlieb, Paul & Vaiman, Michael. (2014). Variations in Eyeball Diameters of the Healthy Adults. Journal of ophthalmology. 2014. 503645. 10.1155/2014/503645. \
    # Average face width is roughly 147.6 mm: DU LL, Wang LM, Zhuang Z. [Measurement and analysis of human head-face dimensions]. Zhonghua Lao Dong Wei Sheng Zhi Ye Bing Za Zhi. 2008 May;26(5):266-70. Chinese. PMID: 18727867.

    # Thus, the width of each eye should be roughly 16% of the face width
    # This function computes eye bounds by extending 16% of the face width upward and leftward from the left eye and the same downward and rightward from the right eye
    # The extension is used as 16% rather than 8% in either direction to allow cases where the innermost edge of the eye is detected as the eye
    extension = 0.16

    face_bounds = get_subject_bounds(face_info)
    _, _, face_width, _ = face_bounds

    left_higher = is_left_eye_higher(face_info)

    if left_higher:
        top_left = int(face_info['keypoints']['left_eye'][0] - (
            face_width * extension)), int(face_info['keypoints']['left_eye'][1] - (face_width * extension))
        bottom_right = int(face_info['keypoints']['right_eye'][0] + (
            face_width * extension)), int(face_info['keypoints']['right_eye'][1] + (face_width * extension))
    else:
        top_left = int(face_info['keypoints']['left_eye'][0] - (
            face_width * extension)), int(face_info['keypoints']['right_eye'][1] - (face_width * extension))
        bottom_right = int(face_info['keypoints']['right_eye'][0] + (
            face_width * extension)), int(face_info['keypoints']['left_eye'][1] + (face_width * extension))

    return (top_left, bottom_right)


def show_point(image, point):
    image = cv.circle(image, point, radius=10, color=(0, 0, 255), thickness=-1)
    show_img(image)


def show_rect(image, top_left, bottom_right):
    cv.rectangle(
        image, (top_left[0], top_left[1]), (bottom_right[0], bottom_right[1]), (0, 255, 0), 1)
    show_img(image)


def crop_subjects(image):
    faces_info = detect_faces(image)

    im_height, im_width, _ = image.shape

    subjects = []
    bounds_list = []

    if not faces_info:
        return [], []

    for face in faces_info:
        # print(face.score)
        bounds = get_subject_bounds(face)

        xmin = bounds.xmin if bounds.xmin > 0 else 0
        xmin = xmin if xmin < 1 else 1
        ymin = bounds.ymin if bounds.ymin > 0 else 0
        ymin = ymin if ymin < 1 else 1

        x, y, w, h = int(xmin * im_width), int(ymin * im_height), int(
            bounds.width * im_width), int(bounds.height * im_height)
        sub = image[y:y + h, x:x + w]

        if not len(sub) or w < 40 or h < 40:
            continue

        try:
            sub = cv.cvtColor(sub, cv.COLOR_BGR2RGB)
        except:
            pass
            # print(xmin, ymin)
            # print(x, y, w, h)
            # print(sub)

        subjects.append(sub)
        bounds_list.append(bounds)
    return subjects, bounds_list


# def show_bounding_boxes(image, bounding_boxes):
#     for box in bounding_boxes:
#         x, y, w, h = box['box']
#         cv.rectangle(image, (x, y), (x + w, y + h), (0, 255, 0), 5)
#     show_img(image)

def get_eye_frame_from_img(filename):
    img = read_img(filename)
    faces_info = detect_faces(img)
    for face in faces_info:
        yield get_eye_bounds(face)


def main():
    img = read_img('./bestOf/resources/examples/IMG_1663.jpg')
    show_img(img)
    subs = crop_subjects_from_segmented_image(img, n=10)

    for sub in subs:
        show_img(sub)

    # subs, _ = crop_subjects(img)
    # for sub in subs:
    #     show_img(sub)


if __name__ == '__main__':
    main()



import mediapipe as mp
import math
import cv2 as cv
import numpy as np
import bestOf.backend.identifyPeople as identifyPeople
import bestOf.backend.evaluateCentering as evaluateCentering

FACE_MESH_CREATOR = mp.solutions.face_mesh.FaceMesh(
    static_image_mode=True, max_num_faces=1, min_detection_confidence=0.5)


def create_mesh(sub):
    results = FACE_MESH_CREATOR.process(sub)
    return results.multi_face_landmarks


def gaussian_illumination_on_full_subject(image):
    mean, l_vals = pure_luminance_analysis(image)
    stdev = math.sqrt(sum((i - mean) ** 2 for i in l_vals) / len(l_vals))

    probabilities = []
    for x in l_vals:
        factor = 1 / (stdev * math.sqrt(2 * math.pi))
        exp = -1 * ((x - mean) ** 2) / (2 * (stdev ** 2))
        probabilities.append(factor * (math.e ** exp))

    return sum(probabilities) / len(probabilities)


def pure_luminance_analysis(image):
    l_vals = []
    image = cv.cvtColor(image, cv.COLOR_RGB2HLS)
    for row in image:
        for pixel in row:
            luminance = pixel[1]
            l_vals.append(luminance)
    mean = sum(l_vals) / len(l_vals)
    return mean, l_vals


def normalize_lighting_scores(group_scores):
    max_val = max(group_scores)

    if not max_val:
        return [1.0] * len(group_scores)

    normalized_scores = [float(score) / float(max_val)
                         for score in group_scores]
    return normalized_scores


def evaluate_lighting_in_region(pixels):
    pass


def split_face(face, num_regions=8):
    pass


def collect_regions(landmarks):
    for landmark in landmarks:
        coord1 = (landmark.x, landmark.y)
        # max dist is sqrt(2), so initialize to an arbitrary number larger than sqrt(2)
        min_dist = 2
        min_dist_2 = 2
        min_dist_landmark = None
        min_dist_landmark_2 = None
        for other in landmarks:
            if landmark != other:
                coord2 = (other.x, other.y)
                dist = evaluateCentering.euclidean(coord1, coord2)
                pass


def evaluate_lighting(image):
    return pure_luminance_analysis(image)[0]


if __name__ == '__main__':
    subs, _ = identifyPeople.crop_subjects(identifyPeople.read_img(
        './bestOf/resources/examples/dark.jpg'))

    x, _ = pure_luminance_analysis(subs[0])
    # print(x)

    subs, _ = identifyPeople.crop_subjects(identifyPeople.read_img(
        './bestOf/resources/examples/bright.jpg'))

    x, _ = pure_luminance_analysis(subs[0])
    # print(x)

    # print(create_mesh(subs[0]))



import os
import shutil


def main():
    lfw_path = '../resources/lfw/'
    dest_path = '../resources/blinkset/nonblinkers/'
    for folder_name in os.listdir(lfw_path):
        # sample only from people who have multiple images
        if len(list(os.listdir(lfw_path + str(folder_name) + '/'))) >= 2:
            images = os.listdir(lfw_path + str(folder_name))
            # sample the first image of that person
            shutil.copy(lfw_path + str(folder_name) +
                        '/' + str(images[0]), dest_path)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'analyzing.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AnalyzingPage(QtCore.QObject):
    def setupUi(self, AnalyzingPage):
        self.AnalyzingPage = AnalyzingPage
        AnalyzingPage.setObjectName("AnalyzingPage")
        AnalyzingPage.resize(730, 612)
        AnalyzingPage.setStyleSheet("color: #1d1e1f;")
        self.verticalLayout = QtWidgets.QVBoxLayout(AnalyzingPage)
        self.verticalLayout.setSpacing(15)
        self.verticalLayout.setObjectName("verticalLayout")
        self.EvaluatingCriteria = QtWidgets.QLabel(AnalyzingPage)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.EvaluatingCriteria.sizePolicy().hasHeightForWidth())
        self.EvaluatingCriteria.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.EvaluatingCriteria.setFont(font)
        self.EvaluatingCriteria.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.EvaluatingCriteria.setObjectName("EvaluatingCriteria")
        self.verticalLayout.addWidget(
            self.EvaluatingCriteria, 0, QtCore.Qt.AlignHCenter | QtCore.Qt.AlignBottom)
        self.Image = QtWidgets.QLabel(AnalyzingPage)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Image.sizePolicy().hasHeightForWidth())
        self.Image.setSizePolicy(sizePolicy)
        self.Image.setText("")
        self.Image.setObjectName("Image")
        self.verticalLayout.addWidget(
            self.Image, 0, QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
        self.spacer = QtWidgets.QLabel(AnalyzingPage)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.spacer.sizePolicy().hasHeightForWidth())
        self.spacer.setSizePolicy(sizePolicy)
        self.spacer.setText("")
        self.spacer.setObjectName("spacer")
        self.verticalLayout.addWidget(self.spacer)

        self.retranslateUi(AnalyzingPage)
        QtCore.QMetaObject.connectSlotsByName(AnalyzingPage)

        self.imagePath = ""

        AnalyzingPage.installEventFilter(self)

    def retranslateUi(self, AnalyzingPage):
        _translate = QtCore.QCoreApplication.translate
        AnalyzingPage.setWindowTitle(_translate("AnalyzingPage", "Form"))
        self.EvaluatingCriteria.setText(_translate(
            "AnalyzingPage", "Evaluating <criteria>"))

    def setCriteria(self, criteria):
        self.EvaluatingCriteria.setText("Analyzing %s ..." % criteria)

    def setImage(self, path):
        self.imagePath = path
        height = self.AnalyzingPage.height() / 1.5
        self.Image.setPixmap(QtGui.QPixmap(path).scaled(
            height, height, QtCore.Qt.KeepAspectRatio))

    def eventFilter(self, obj, event):
        if (event.type() == QtCore.QEvent.Resize and self.imagePath != ""):
            self.setImage(self.imagePath)
        return super().eventFilter(obj, event)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    AnalyzingPage = QtWidgets.QWidget()
    ui = Ui_AnalyzingPage()
    ui.setupUi(AnalyzingPage)
    AnalyzingPage.show()
    sys.exit(app.exec_())



import torch
import torch.nn as nn
import torch.nn.functional as fn


class BlinkAndCropNet(torch.nn.Module):
    def __init__(self):
        super(BlinkAndCropNet, self).__init__()
        self.conv1 = nn.Conv2d(3, 32, 3)
        self.conv2 = nn.Conv2d(32, 32, 3)
        self.pool1 = nn.MaxPool2d(2)

        self.drop1 = nn.Dropout2d(0.25)

        self.conv3 = nn.Conv2d(32, 64, 3)
        self.pool2 = nn.MaxPool2d(2)

        self.drop2 = nn.Dropout2d(0.25)

        self.dense1 = nn.Linear(20736, 128)
        self.dense2 = nn.Linear(128, 64)
        self.dense3 = nn.Linear(64, 1)
        self.sigmoid = nn.Sigmoid()

    def forward(self, x):
        x = fn.relu(self.conv2(fn.relu(self.conv1(x))))
        x = self.drop1(self.pool1(x))

        x = fn.relu(self.conv3(x))
        x = self.drop2(self.pool2(x))
        x = torch.flatten(x, 1)

        x = fn.relu(self.dense1(x))
        x = fn.relu(self.dense2(x))
        x = self.dense3(x)
        x = self.sigmoid(x)
        return x


class cropped_model(torch.nn.Module):
    def __init__(self):
        super(cropped_model, self).__init__()
        self.conv1 = nn.Conv2d(3, 32, 3)
        self.conv2 = nn.Conv2d(32, 32, 3)
        self.pool1 = nn.MaxPool2d(2)

        self.drop1 = nn.Dropout2d(0.25)

        self.conv3 = nn.Conv2d(32, 64, 3)
        self.pool2 = nn.MaxPool2d(2)

        self.drop2 = nn.Dropout2d(0.25)

        self.dense1 = nn.Linear(20736, 128)
        self.dense2 = nn.Linear(128, 64)
        self.dense3 = nn.Linear(64, 1)
        self.sigmoid = nn.Sigmoid()

    def forward(self, x):
        x = fn.relu(self.conv2(fn.relu(self.conv1(x))))
        x = self.drop1(self.pool1(x))

        x = fn.relu(self.conv3(x))
        x = self.drop2(self.pool2(x))
        x = torch.flatten(x, 1)

        x = fn.relu(self.dense1(x))
        x = fn.relu(self.dense2(x))
        x = self.dense3(x)
        x = self.sigmoid(x)
        return x


class custom_dataset(torch.utils.data.Dataset):
    def __init__(self, all_x, all_y, transform, target_transform=None):
        self.all_x = all_x
        self.all_y = all_y
        self.transform = transform
        self.target_transform = target_transform

    def __len__(self):
        return len(self.all_x)

    def __getitem__(self, index):
        image = self.all_x[index]
        label = self.all_y[index]
        image = self.transform(image)

        if self.target_transform:
            label = self.target_transform(label)

        return image, label



from PyQt5 import QtWidgets
from PyQt5 import QtCore
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
import sys
import numpy as np
import json
import os
print("Starting...")

sys.path.insert(1, 'bestOf/backend')


import bestOf.backend.similarity as similarity
from bestOf.backend.classDefinitions import BlinkAndCropNet, cropped_model
import bestOf.backend.blinkDetector as blinkDetector
import bestOf.backend.cropDetector as cropDetector
import bestOf.backend.evaluateSharpness as evaluateSharpness
import bestOf.backend.identifyPeople as identifyPeople
import bestOf.backend.evaluateCentering as evaluateCentering
import bestOf.backend.sitePackagePathConstructor as sitePackagePathConstructor


def loadImages(filenames):
    for filename in filenames:
        image = identifyPeople.read_img(filename)
        yield (image * 255).astype(np.uint8)[:, :, :3] if 'png' in filename else image


def read_settings():
    path = sitePackagePathConstructor.get_site_package_path(
        './bestOf/backend/settings.json')
    settings = {}
    with open(path, 'r') as settings_file:
        settings = json.load(settings_file)
    return settings


def write_settings(settings):
    path = sitePackagePathConstructor.get_site_package_path(
        './bestOf/backend/settings.json')
    with open(path, 'w') as settings_file:
        json.dump(settings, settings_file)


IMAGELIST = []
GROUPS = []

# Reference Source 1: https://learndataanalysis.org/how-to-pass-data-from-one-window-to-another-pyqt5-tutorial/


def main():
    class processingResults(QWidget):
        def __init__(self):
            super().__init__()
            self.options()
            self.setWindowTitle("Best Of")
            self.setGeometry(1550, 800, 500, 500)
            self.layout = QVBoxLayout()
            self.layout.addWidget(self.des)
            self.setLayout(self.layout)

        def display(self):
            self.__init__()
            self.show()

        def options(self):
            self.des = QGroupBox("Images Selected.",
                                 alignment=QtCore.Qt.AlignCenter)
            vert = QVBoxLayout()
            layout = QHBoxLayout()
            self.redo = QPushButton('&Run Again')
            self.redo.clicked.connect(self.imageProcessing)
            layout.addWidget(self.redo)
            self.back = QPushButton('&Back to Main Menu')
            self.back.clicked.connect(self.close)
            layout.addWidget(self.back)
            info = IMAGELIST
            groups = GROUPS
            if len(info) == 0:
                return
            # print(info)
            # print(groups)
            for group in groups:
                h = QHBoxLayout()
                for index in group:
                    lab = QLabel()
                    for el in info:
                        if el[0] == index:
                            lab.setPixmap(QPixmap(el[1]))
                    h.addWidget(lab)
                vert.addLayout(h)
            vert.addLayout(layout)
            self.des.setLayout(vert)

        def imageProcessing(self):
            image_generator = loadImages([element[1] for element in IMAGELIST])

            # Criteria should be populated from settings selections
            criteria = {
                'sharpness': 1,
                'centering': 1,
                'lighting': 0,
                'resolution': 0
            }

            if criteria['sharpness']:
                sharpness_scores = []
                for image in image_generator:
                    sharpness_scores.append(evaluateSharpness.evaluate_sharpness(
                        image))
                # index to normalized sharpness score + half of normalized subject sharpness score
                sharpness_map = {}
                for group in GROUPS:
                    group_sharpness_scores = []
                    for index in group:
                        for pos, item in enumerate(IMAGELIST):
                            if item[0] == index:
                                group_sharpness_scores.append(
                                    sharpness_scores[pos])
                                break
                    group_sharpness_scores = evaluateSharpness.normalize_sharpness_scores(
                        group_sharpness_scores)

                    avg_subject_sharpness_scores = []
                    for index in group:
                        for pos, item in enumerate(IMAGELIST):
                            if item[0] == index:
                                subject_sharpness_scores = item[3]
                                if not len(subject_sharpness_scores):
                                    avg_subject_sharpness_scores.append(0)
                                    continue

                                avg_subject_sharpness_scores.append(
                                    sum(subject_sharpness_scores) / len(subject_sharpness_scores))
                    avg_subject_sharpness_scores = evaluateSharpness.normalize_sharpness_scores(
                        avg_subject_sharpness_scores)

                    for idx_in_group, index in enumerate(group):
                        sharpness_map[index] = group_sharpness_scores[idx_in_group] + \
                            (avg_subject_sharpness_scores[idx_in_group] / 2)

                print(sharpness_map)

            if criteria['centering']:
                pass

    class settingsMenu(QWidget):
        def __init__(self):
            super().__init__()
            self.options()
            self.setWindowTitle("Settings Menu")
            self.setGeometry(1550, 800, 500, 500)
            self.layout = QVBoxLayout()
            self.layout.addWidget(self.des)
            self.setLayout(self.layout)

        def display(self):
            self.show()

        def options(self):
            self.des = QGroupBox("Apply the following criteria for image refinement.",
                                 alignment=QtCore.Qt.AlignCenter)
            layout = QHBoxLayout()

            self.closeButton = QPushButton('&Close Settings')
            self.closeButton.clicked.connect(self.close)
            layout.addWidget(self.closeButton)
            self.des.setLayout(layout)

    class mainWindow(QWidget):
        def __init__(self):
            super().__init__()
            self.setWindowTitle("Best Of: Main Menu")
            self.resize(800, 800)
            self.setGeometry(1550, 800, 500, 500)
            self.nextWindow = settingsMenu()
            self.process = processingResults()

            self.criteria_settings = read_settings()
            print(self.criteria_settings)

            # self.progress = QProgressBar(self)
            # self.progress.setGeometry()
            self.makeUI()
            self.layout = QVBoxLayout()
            self.layout.addWidget(self.firstText)
            self.setLayout(self.layout)

        def closeEvent(self, event):
            write_settings(self.criteria_settings)

        def makeUI(self):
            self.firstText = QGroupBox("Welcome, browse your computer for images to start picture analysis.",
                                       alignment=QtCore.Qt.AlignCenter)
            layout = QHBoxLayout()
            self.addImage = QPushButton('&Add Images')
            self.addImage.clicked.connect(self.getFiles)
            layout.addWidget(self.addImage)
            self.processImage = QPushButton('&Process Image(s)')
            self.processImage.clicked.connect(self.process.display)
            layout.addWidget(self.processImage)
            self.settings = QPushButton('&More Settings')
            self.settings.clicked.connect(self.nextWindow.display)
            layout.addWidget(self.settings)
            self.firstText.setLayout(layout)

        # Grabs the files for image selection... work in progress...

        def getFiles(self):
            global IMAGELIST
            global GROUPS
            file = QFileDialog.getOpenFileNames(
                self, 'Add Files', QtCore.QDir.rootPath())
            files = ""
            layout = QVBoxLayout
            image_generator = loadImages(list(file[0]))
            vectors = []

            # self.criteria_settings['sharpness'] = 2
            # self.criteria_settings['resolution'] = 3

            for image in image_generator:
                v = similarity.generate_feature_vector(image)
                vectors.append(v)

            threshold = 0.8  # this should be grabbed from whatever the user set it to in the settings
            groups = similarity.group(vectors, threshold=threshold)
            # print(groups)
            GROUPS = groups

            image_generator = loadImages(list(file[0]))

            default_scores = []
            subject_sharpness_scores = []
            centering_scores = []

            for image in image_generator:
                # print('Scanning Image...')
                subjects, bounds_list = identifyPeople.crop_subjects(image)
                # print("len of subjects", len(subjects))
                blinks = 0
                crops = 0

                subject_sharpness_scores.append([])

                centering_scores.append(
                    evaluateCentering.evaluate_centering(bounds_list, image))

                for sub in subjects:
                    # identifyPeople.show_img(sub)
                    # print('Scanning Subject...')
                    if blinkDetector.test(sub):
                        blinks += 1
                    if cropDetector.test(sub):
                        crops += 1

                    subject_sharpness_scores[-1].append(
                        evaluateSharpness.evaluate_sharpness(sub))

                if len(subjects) == 0:
                    default_scores.append(0)
                    continue

                blink_score = (len(subjects) - blinks) / len(subjects)
                crop_score = (len(subjects) - crops) / len(subjects)

                default_score = blink_score + crop_score
                default_scores.append(default_score)

            if len(IMAGELIST):
                max_index = max([ind for ind in IMAGELIST[0]])
            else:
                max_index = -1

            final = zip(
                range(max_index + 1, max_index + 1 + len(file[0])), file[0], default_scores, subject_sharpness_scores, centering_scores)

            if len(IMAGELIST):
                final = final + IMAGELIST
            final = sorted(final, key=lambda x: x[2], reverse=True)
            print(final)
            IMAGELIST = final

    newApp = QApplication(sys.argv)  # Creates application class
    wind = mainWindow()
    wind.show()
    newApp.exec()  # Executes app with the window


if __name__ == '__main__':
    main()



import math


def euclidean(coord1, coord2, y_modifier=3):
    return math.sqrt(((coord1[0] - coord2[0]) ** 2) + (((coord1[1] - coord2[1]) / y_modifier) ** 2))


def evaluate_centering(bounds_list, image):
    im_height, im_width, _ = image.shape
    center = (im_width / 2, im_height / 2)
    total_dist = 0

    # print(bounds_list)

    for bounds in bounds_list:
        x, y, w, h = int(bounds.xmin * im_width), int(bounds.ymin * im_height), int(
            bounds.width * im_width), int(bounds.height * im_height)
        coord = (x + w, y + h)
        dist = euclidean(coord, center)
        total_dist += dist

    # print('Total', total_dist)

    max_dist = math.sqrt((center[0] ** 2) + (center[1] ** 2))
    # print('Max', max_dist)

    return (max_dist - (total_dist / len(bounds_list))) / max_dist if len(bounds_list) and max_dist else None


def normalize_centering_scores(group_scores):
    max_val = max(group_scores)

    if not max_val:
        return [1.0] * len(group_scores)

    normalized_scores = [float(score) / float(max_val)
                         for score in group_scores]
    return normalized_scores



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Settings(QtCore.QObject):
    criteriaChangedSignal = QtCore.pyqtSignal(str, int)
    thresholdChangedSignal = QtCore.pyqtSignal(float)

    def setupUi(self, Settings):
        Settings.setObjectName("Settings")
        Settings.resize(898, 603)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Settings.sizePolicy().hasHeightForWidth())
        Settings.setSizePolicy(sizePolicy)
        Settings.setStyleSheet("QFrame {\n"
                               "color:  #00ffff;\n"
                               "}\n"
                               "QLabel, QPushButton {\n"
                               "color: #1d1e1f;\n"
                               "}\n"
                               "\n"
                               "")
        self.verticalLayout = QtWidgets.QVBoxLayout(Settings)
        self.verticalLayout.setObjectName("verticalLayout")
        self.Back = QtWidgets.QPushButton(Settings)
        self.Back.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Back.sizePolicy().hasHeightForWidth())
        self.Back.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.Back.setFont(font)
        self.Back.setStyleSheet("border: bold")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/img/back-button.png"),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Back.setIcon(icon)
        self.Back.setIconSize(QtCore.QSize(50, 50))
        self.Back.setObjectName("Back")
        self.verticalLayout.addWidget(
            self.Back, 0, QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
        spacerItem = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.frame = QtWidgets.QFrame(Settings)
        self.frame.setStyleSheet("QRadioButton::indicator {\n"
                                 "    width:10px;\n"
                                 "    height:10px;\n"
                                 "    border-radius: 8px;\n"
                                 "    border: 3px solid #57AECB;\n"
                                 "}\n"
                                 "\n"
                                 "QRadioButton::indicator:unchecked  {\n"
                                 "    background: transparent;\n"
                                 "}\n"
                                 "\n"
                                 "QRadioButton::indicator:checked  {\n"
                                 "    background: #87CEEB;\n"
                                 "}")
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setMinimumSize(QtCore.QSize(880, 290))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.horizontalSlider = QtWidgets.QSlider(self.frame)
        self.horizontalSlider.setGeometry(QtCore.QRect(220, 15, 641, 22))
        self.horizontalSlider.setStyleSheet("QSlider::groove:horizontal {\n"
                                            "    border: 1px solid #999999;\n"
                                            "    height: 4px;\n"
                                            "    margin:0px 0;\n"
                                            "}\n"
                                            "\n"
                                            "QSlider::handle:horizontal {\n"
                                            "    background: #87CEEB;\n"
                                            "    border: 3px solid #57AECB;\n"
                                            "    width: 10px;\n"
                                            "    margin: -6px 0; \n"
                                            "    border-radius: 8px;\n"
                                            "}\n"
                                            "\n"
                                            "QSlider::sub-page:horizontal {\n"
                                            "    height: 1px;\n"
                                            "    background:  #87CEEB;\n"
                                            "}")
        self.horizontalSlider.setMinimum(70)
        self.horizontalSlider.setMaximum(95)
        self.horizontalSlider.setProperty("value", 80)
        self.horizontalSlider.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider.setObjectName("horizontalSlider")
        self.Threshhold = QtWidgets.QLabel(self.frame)
        self.Threshhold.setGeometry(QtCore.QRect(10, 10, 187, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.Threshhold.setFont(font)
        self.Threshhold.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Threshhold.setObjectName("Threshhold")
        self.Important = QtWidgets.QLabel(self.frame)
        self.Important.setGeometry(QtCore.QRect(572, 50, 94, 113))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Important.sizePolicy().hasHeightForWidth())
        self.Important.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Important.setFont(font)
        self.Important.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Important.setObjectName("Important")
        self.SomewhatImp = QtWidgets.QLabel(self.frame)
        self.SomewhatImp.setGeometry(QtCore.QRect(372, 50, 149, 113))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.SomewhatImp.sizePolicy().hasHeightForWidth())
        self.SomewhatImp.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.SomewhatImp.setFont(font)
        self.SomewhatImp.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.SomewhatImp.setObjectName("SomewhatImp")
        self.VeryImportant = QtWidgets.QLabel(self.frame)
        self.VeryImportant.setGeometry(QtCore.QRect(720, 50, 143, 113))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.VeryImportant.sizePolicy().hasHeightForWidth())
        self.VeryImportant.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.VeryImportant.setFont(font)
        self.VeryImportant.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.VeryImportant.setObjectName("VeryImportant")
        self.DoNotConsider = QtWidgets.QLabel(self.frame)
        self.DoNotConsider.setGeometry(QtCore.QRect(190, 50, 147, 113))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.DoNotConsider.sizePolicy().hasHeightForWidth())
        self.DoNotConsider.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.DoNotConsider.setFont(font)
        self.DoNotConsider.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.DoNotConsider.setObjectName("DoNotConsider")
        self.Sharpness = QtWidgets.QLabel(self.frame)
        self.Sharpness.setGeometry(QtCore.QRect(30, 135, 147, 31))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Sharpness.sizePolicy().hasHeightForWidth())
        self.Sharpness.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Sharpness.setFont(font)
        self.Sharpness.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Sharpness.setObjectName("Sharpness")
        self.Centering = QtWidgets.QLabel(self.frame)
        self.Centering.setGeometry(QtCore.QRect(30, 175, 147, 31))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Centering.sizePolicy().hasHeightForWidth())
        self.Centering.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Centering.setFont(font)
        self.Centering.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Centering.setObjectName("Centering")
        self.Lighting = QtWidgets.QLabel(self.frame)
        self.Lighting.setGeometry(QtCore.QRect(30, 215, 147, 31))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Lighting.sizePolicy().hasHeightForWidth())
        self.Lighting.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Lighting.setFont(font)
        self.Lighting.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Lighting.setObjectName("Lighting")
        self.Resolution = QtWidgets.QLabel(self.frame)
        self.Resolution.setGeometry(QtCore.QRect(30, 255, 147, 31))
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Resolution.sizePolicy().hasHeightForWidth())
        self.Resolution.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Resolution.setFont(font)
        self.Resolution.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Resolution.setObjectName("Resolution")
        self.line = QtWidgets.QFrame(self.frame)
        self.line.setGeometry(QtCore.QRect(0, 120, 881, 20))
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.line_2 = QtWidgets.QFrame(self.frame)
        self.line_2.setGeometry(QtCore.QRect(0, 160, 881, 20))
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setObjectName("line_2")
        self.line_3 = QtWidgets.QFrame(self.frame)
        self.line_3.setGeometry(QtCore.QRect(0, 200, 881, 20))
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setObjectName("line_3")
        self.line_4 = QtWidgets.QFrame(self.frame)
        self.line_4.setGeometry(QtCore.QRect(0, 240, 881, 20))
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setObjectName("line_4")
        self.line_5 = QtWidgets.QFrame(self.frame)
        self.line_5.setGeometry(QtCore.QRect(160, 90, 20, 201))
        self.line_5.setStyleSheet("")
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setLineWidth(1)
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setObjectName("line_5")
        self.line_6 = QtWidgets.QFrame(self.frame)
        self.line_6.setGeometry(QtCore.QRect(350, 90, 20, 201))
        self.line_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_6.setLineWidth(1)
        self.line_6.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_6.setObjectName("line_6")
        self.line_7 = QtWidgets.QFrame(self.frame)
        self.line_7.setGeometry(QtCore.QRect(540, 90, 20, 201))
        self.line_7.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_7.setLineWidth(1)
        self.line_7.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_7.setObjectName("line_7")
        self.line_8 = QtWidgets.QFrame(self.frame)
        self.line_8.setGeometry(QtCore.QRect(690, 90, 20, 201))
        self.line_8.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_8.setLineWidth(1)
        self.line_8.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_8.setObjectName("line_8")
        self.frame_6 = QtWidgets.QFrame(self.frame)
        self.frame_6.setGeometry(QtCore.QRect(0, 90, 880, 200))
        self.frame_6.setStyleSheet("QFrame {\n"
                                   "    border: 1px solid #b187eb;\n"
                                   "}")
        self.frame_6.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_6.setObjectName("frame_6")

        self.SharpnessButtons = QtWidgets.QFrame(self.frame)
        self.SharpnessButtons.setGeometry(QtCore.QRect(253, 143, 540, 20))
        self.SharpnessButtons.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.SharpnessButtons.setFrameShadow(QtWidgets.QFrame.Raised)
        self.SharpnessButtons.setObjectName("SharpnessButtons")
        self.sharpness_3 = QtWidgets.QRadioButton(self.SharpnessButtons)
        self.sharpness_3.setGeometry(QtCore.QRect(520, 0, 16, 17))
        self.sharpness_3.setText("")
        self.sharpness_3.setChecked(False)
        self.sharpness_3.setObjectName("sharpness_3")
        self.sharpness_0 = QtWidgets.QRadioButton(self.SharpnessButtons)
        self.sharpness_0.setGeometry(QtCore.QRect(0, 0, 16, 17))
        self.sharpness_0.setText("")
        self.sharpness_0.setChecked(True)
        self.sharpness_0.setObjectName("sharpness_0")
        self.sharpness_1 = QtWidgets.QRadioButton(self.SharpnessButtons)
        self.sharpness_1.setGeometry(QtCore.QRect(190, 0, 16, 17))
        self.sharpness_1.setText("")
        self.sharpness_1.setChecked(False)
        self.sharpness_1.setObjectName("sharpness_1")
        self.sharpness_2 = QtWidgets.QRadioButton(self.SharpnessButtons)
        self.sharpness_2.setGeometry(QtCore.QRect(360, 0, 16, 17))
        self.sharpness_2.setText("")
        self.sharpness_2.setChecked(False)
        self.sharpness_2.setObjectName("sharpness_2")
        self.CenteringButtons = QtWidgets.QFrame(self.frame)
        self.CenteringButtons.setGeometry(QtCore.QRect(253, 183, 540, 20))
        self.CenteringButtons.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.CenteringButtons.setFrameShadow(QtWidgets.QFrame.Raised)
        self.CenteringButtons.setObjectName("CenteringButtons")
        self.centering_3 = QtWidgets.QRadioButton(self.CenteringButtons)
        self.centering_3.setGeometry(QtCore.QRect(520, 0, 16, 17))
        self.centering_3.setText("")
        self.centering_3.setChecked(False)
        self.centering_3.setObjectName("centering_3")
        self.centering_0 = QtWidgets.QRadioButton(self.CenteringButtons)
        self.centering_0.setGeometry(QtCore.QRect(0, 0, 16, 17))
        self.centering_0.setText("")
        self.centering_0.setChecked(True)
        self.centering_0.setObjectName("centering_0")
        self.centering_1 = QtWidgets.QRadioButton(self.CenteringButtons)
        self.centering_1.setGeometry(QtCore.QRect(190, 0, 16, 17))
        self.centering_1.setText("")
        self.centering_1.setChecked(False)
        self.centering_1.setObjectName("centering_1")
        self.centering_2 = QtWidgets.QRadioButton(self.CenteringButtons)
        self.centering_2.setGeometry(QtCore.QRect(360, 0, 16, 17))
        self.centering_2.setText("")
        self.centering_2.setChecked(False)
        self.centering_2.setObjectName("centering_2")
        self.LightingButtons = QtWidgets.QFrame(self.frame)
        self.LightingButtons.setGeometry(QtCore.QRect(253, 223, 540, 20))
        self.LightingButtons.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.LightingButtons.setFrameShadow(QtWidgets.QFrame.Raised)
        self.LightingButtons.setObjectName("LightingButtons")
        self.lighting_3 = QtWidgets.QRadioButton(self.LightingButtons)
        self.lighting_3.setGeometry(QtCore.QRect(520, 0, 16, 17))
        self.lighting_3.setText("")
        self.lighting_3.setChecked(False)
        self.lighting_3.setObjectName("lighting_3")
        self.lighting_0 = QtWidgets.QRadioButton(self.LightingButtons)
        self.lighting_0.setGeometry(QtCore.QRect(0, 0, 16, 17))
        self.lighting_0.setText("")
        self.lighting_0.setChecked(True)
        self.lighting_0.setObjectName("lighting_0")
        self.lighting_1 = QtWidgets.QRadioButton(self.LightingButtons)
        self.lighting_1.setGeometry(QtCore.QRect(190, 0, 16, 17))
        self.lighting_1.setText("")
        self.lighting_1.setChecked(False)
        self.lighting_1.setObjectName("lighting_1")
        self.lighting_2 = QtWidgets.QRadioButton(self.LightingButtons)
        self.lighting_2.setGeometry(QtCore.QRect(360, 0, 16, 17))
        self.lighting_2.setText("")
        self.lighting_2.setChecked(False)
        self.lighting_2.setObjectName("lighting_2")
        self.ResolutionButtons = QtWidgets.QFrame(self.frame)
        self.ResolutionButtons.setGeometry(QtCore.QRect(253, 263, 540, 20))
        self.ResolutionButtons.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.ResolutionButtons.setFrameShadow(QtWidgets.QFrame.Raised)
        self.ResolutionButtons.setObjectName("ResolutionButtons")
        self.resolution_3 = QtWidgets.QRadioButton(self.ResolutionButtons)
        self.resolution_3.setGeometry(QtCore.QRect(520, 0, 16, 17))
        self.resolution_3.setText("")
        self.resolution_3.setChecked(False)
        self.resolution_3.setObjectName("resolution_3")
        self.resolution_0 = QtWidgets.QRadioButton(self.ResolutionButtons)
        self.resolution_0.setGeometry(QtCore.QRect(0, 0, 16, 17))
        self.resolution_0.setText("")
        self.resolution_0.setChecked(True)
        self.resolution_0.setObjectName("resolution_0")
        self.resolution_1 = QtWidgets.QRadioButton(self.ResolutionButtons)
        self.resolution_1.setGeometry(QtCore.QRect(190, 0, 16, 17))
        self.resolution_1.setText("")
        self.resolution_1.setChecked(False)
        self.resolution_1.setObjectName("resolution_1")
        self.resolution_2 = QtWidgets.QRadioButton(self.ResolutionButtons)
        self.resolution_2.setGeometry(QtCore.QRect(360, 0, 16, 17))
        self.resolution_2.setText("")
        self.resolution_2.setChecked(False)
        self.resolution_2.setObjectName("resolution_2")
        self.verticalLayout.addWidget(self.frame, 0, QtCore.Qt.AlignHCenter)
        spacerItem1 = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(Settings)
        QtCore.QMetaObject.connectSlotsByName(Settings)

        self.sharpnessButtonGroup = QtWidgets.QButtonGroup()
        self.sharpnessButtonGroup.addButton(self.sharpness_0, 0)
        self.sharpnessButtonGroup.addButton(self.sharpness_1, 1)
        self.sharpnessButtonGroup.addButton(self.sharpness_2, 2)
        self.sharpnessButtonGroup.addButton(self.sharpness_3, 3)
        self.sharpnessButtonGroup.buttonClicked.connect(
            self.onSharpnessChanged)

        self.centeringButtonGroup = QtWidgets.QButtonGroup()
        self.centeringButtonGroup.addButton(self.centering_0, 0)
        self.centeringButtonGroup.addButton(self.centering_1, 1)
        self.centeringButtonGroup.addButton(self.centering_2, 2)
        self.centeringButtonGroup.addButton(self.centering_3, 3)
        self.centeringButtonGroup.buttonClicked.connect(
            self.onCenteringChanged)

        self.lightingButtonGroup = QtWidgets.QButtonGroup()
        self.lightingButtonGroup.addButton(self.lighting_0, 0)
        self.lightingButtonGroup.addButton(self.lighting_1, 1)
        self.lightingButtonGroup.addButton(self.lighting_2, 2)
        self.lightingButtonGroup.addButton(self.lighting_3, 3)
        self.lightingButtonGroup.buttonClicked.connect(
            self.onLightingChanged)

        self.resolutionButtonGroup = QtWidgets.QButtonGroup()
        self.resolutionButtonGroup.addButton(self.resolution_0, 0)
        self.resolutionButtonGroup.addButton(self.resolution_1, 1)
        self.resolutionButtonGroup.addButton(self.resolution_2, 2)
        self.resolutionButtonGroup.addButton(self.resolution_3, 3)
        self.resolutionButtonGroup.buttonClicked.connect(
            self.onResolutionChanged)

        self.horizontalSlider.valueChanged.connect(self.onThresholdChanged)

    def retranslateUi(self, Settings):
        _translate = QtCore.QCoreApplication.translate
        Settings.setWindowTitle(_translate("Settings", "Form"))
        self.Back.setText(_translate("Settings", " Back"))
        self.Threshhold.setText(_translate(
            "Settings", "Similarity threshold:"))
        self.Important.setText(_translate("Settings", "Important"))
        self.SomewhatImp.setText(_translate("Settings", "Somewhat Imp."))
        self.VeryImportant.setText(_translate("Settings", "Very Important"))
        self.DoNotConsider.setText(_translate("Settings", "Do not consider"))
        self.Sharpness.setText(_translate("Settings", "Sharpness"))
        self.Centering.setText(_translate("Settings", "Centering"))
        self.Lighting.setText(_translate("Settings", "Lighting"))
        self.Resolution.setText(_translate("Settings", "Resolution"))

    def setSettings(self, settings):
        self.sharpnessButtonGroup.button(
            settings["sharpness"]).setChecked(True)
        self.centeringButtonGroup.button(
            settings["centering"]).setChecked(True)
        self.lightingButtonGroup.button(
            settings["lighting"]).setChecked(True)
        self.resolutionButtonGroup.button(
            settings["resolution"]).setChecked(True)
        self.horizontalSlider.setValue(int(settings["threshold"] * 100))

    def onSharpnessChanged(self, button):
        if button.isChecked():
            self.criteriaChangedSignal.emit(
                "sharpness", self.sharpnessButtonGroup.checkedId())

    def onCenteringChanged(self, button):
        if button.isChecked():
            self.criteriaChangedSignal.emit(
                "centering", self.centeringButtonGroup.checkedId())

    def onLightingChanged(self, button):
        if button.isChecked():
            self.criteriaChangedSignal.emit(
                "lighting", self.lightingButtonGroup.checkedId())

    def onResolutionChanged(self, button):
        if button.isChecked():
            self.criteriaChangedSignal.emit(
                "resolution", self.resolutionButtonGroup.checkedId())

    def onThresholdChanged(self, value):
        self.thresholdChangedSignal.emit(value / 100)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Settings = QtWidgets.QWidget()
    ui = Ui_Settings()
    ui.setupUi(Settings)
    Settings.show()
    sys.exit(app.exec_())



import cv2 as cv
import numpy as np
import bestOf.backend.identifyPeople as identifyPeople


def apply_kernel(image, a=-4, b=1, c=0):
    image = cv.cvtColor(image, cv.COLOR_RGB2GRAY)
    kernel = np.array(([c, b, c], [b, a, b], [c, b, c]), dtype='int')

    result = cv.filter2D(image, -1, kernel)
    # identifyPeople.show_img(result)

    return result


def get_sharpness_score(transformed_image):
    # A sharp image will have a lot of variation
    # A blurry image will be uniform in more places, meaning its variance will be lower
    return transformed_image.var()


def evaluate_sharpness(image):
    return get_sharpness_score(apply_kernel(image))


def normalize_sharpness_scores(group_scores):
    max_val = max(group_scores)

    if not max_val:
        return [1.0] * len(group_scores)

    normalized_scores = [float(score) / float(max_val)
                         for score in group_scores]
    return normalized_scores


if __name__ == '__main__':
    img = identifyPeople.read_img(
        './bestOf/resources/examples/IMG_1667.jpg')
    s = evaluate_sharpness(img)
    # print(s)
    img = identifyPeople.read_img(
        './bestOf/resources/examples/IMG_1668.jpg')
    s = evaluate_sharpness(img)
    # print(s)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'app1.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(764, 596)
        MainWindow.setStyleSheet(
            "QMainWindow{background: white;}\n"
            "color: #1d1e1f;")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(
            QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout.setContentsMargins(-1, 0, -1, 5)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setObjectName("verticalLayout")
        self.stackedWidget = QtWidgets.QStackedWidget(self.centralwidget)
        self.stackedWidget.setObjectName("stackedWidget")
        self.verticalLayout.addWidget(self.stackedWidget)
        self.line_2 = QtWidgets.QFrame(self.centralwidget)
        self.line_2.setStyleSheet("color: #00ffff;")
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setLineWidth(2)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.line_3 = QtWidgets.QFrame(self.centralwidget)
        self.line_3.setStyleSheet("color: #87CEEB;")
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setLineWidth(2)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setStyleSheet("color: #b187eb;")
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(10, -1, 10, -1)
        self.horizontalLayout.setSpacing(16)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.Status = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        self.Status.setFont(font)
        self.Status.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.Status.setObjectName("Status")
        self.horizontalLayout.addWidget(self.Status)
        self.progressBar = QtWidgets.QProgressBar(self.centralwidget)
        self.progressBar.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.progressBar.setFont(font)
        self.progressBar.setStyleSheet("QProgressBar {\n"
                                       "    border: 2px solid black;\n"
                                       "   text-align: center;\n"
                                       "}\n"
                                       "\n"
                                       "QProgressBar::chunk {\n"
                                       "    background-color: #00ffff;\n"
                                       "    width: 20px;\n"
                                       "}")
        self.progressBar.setProperty("value", 24)
        self.progressBar.setTextVisible(True)
        self.progressBar.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.progressBar.setObjectName("progressBar")
        self.horizontalLayout.addWidget(self.progressBar)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 1, 3, 1)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        self.stackedWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.Status.setText(_translate(
            "MainWindow", "Start by Uploading Images. Click Add Files above!"))

    def changeProgress(self, progress):
        self.progressBar.setProperty("value", progress)
        if progress == 0 or progress == 100:
            self.progressBar.setVisible(False)
        else:
            self.progressBar.setVisible(True)

    def changeStatus(self, status, color="#1d1e1f"):
        self.Status.setText(status)
        self.Status.setStyleSheet("color: %s;" % color)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



def evaluate_resolution(image):
    return image.shape[0] * image.shape[1]


def normalize_resolution_scores(group_scores):
    max_val = max(group_scores)

    if not max_val:
        return [1.0] * len(group_scores)

    normalized_scores = [float(score) / float(max_val)
                         for score in group_scores]
    return normalized_scores



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'results.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.


from PyQt5 import QtCore, QtGui, QtWidgets


def clearLayout(layout):
    if layout is not None:
        while layout.count():
            child = layout.takeAt(0)
            if child.widget() is not None:
                child.widget().deleteLater()
            elif child.layout() is not None:
                clearLayout(child.layout())


class SelectableImage(QtWidgets.QLabel):
    toggled = QtCore.pyqtSignal(bool, int)

    def __init__(self, id, parent=None):
        QtWidgets.QLabel.__init__(self, parent)
        self.selected = False
        self.id = id

    def mousePressEvent(self, event):
        self.selected = not self.selected
        if self.selected:
            self.setStyleSheet("border: 3px solid #87CEEB;")
        else:
            self.setStyleSheet("")
        self.toggled.emit(self.selected, self.id)


class Ui_Results(QtCore.QObject):
    imageToggledSignal = QtCore.pyqtSignal(bool, int)
    downloadSelectedSignal = QtCore.pyqtSignal()

    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(696, 537)
        Form.setStyleSheet("color: #1d1e1f;")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.Back = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.Back.sizePolicy().hasHeightForWidth())
        self.Back.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.Back.setFont(font)
        self.Back.setStyleSheet("border: bold")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/img/back-button.png"),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Back.setIcon(icon)
        self.Back.setIconSize(QtCore.QSize(50, 50))
        self.Back.setObjectName("Back")
        self.verticalLayout_2.addWidget(
            self.Back, 0, QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
        self.scrollArea = QtWidgets.QScrollArea(Form)
        self.scrollArea.setStyleSheet(
            "background: transparent; border: none;")
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 762, 444))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(
            self.scrollAreaWidgetContents)
        self.verticalLayout.setSpacing(15)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_2.addWidget(self.scrollArea)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.Back.setText(_translate("Form", " Back"))

    def display(self, imageList, groups):
        clearLayout(self.verticalLayout)

        # print('Displaying Results...')
        # print(groups)
        # print(imageList)

        for pos, group in enumerate(groups):
            horizontalLayout = QtWidgets.QHBoxLayout()
            horizontalLayout.setSpacing(15)

            label = QtWidgets.QLabel()
            font = QtGui.QFont()
            font.setFamily("MS Shell Dlg 2")
            font.setPointSize(12)
            label.setFont(font)
            label.setText(str(pos + 1))

            horizontalLayout.addWidget(
                label, 0, QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)

            for index in group:
                for item in imageList:
                    if item[0] == index:
                        image = SelectableImage(index)
                        image.setGeometry(0, 0, 200, 200)
                        sizePolicy = QtWidgets.QSizePolicy(
                            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
                        image.setSizePolicy(sizePolicy)
                        image.setPixmap(QtGui.QPixmap(item[1]).scaled(
                            200, 200, QtCore.Qt.KeepAspectRatio))
                        image.toggled.connect(
                            lambda s, i: self.imageToggledSignal.emit(s, i))
                        horizontalLayout.addWidget(image)

            spacer = QtWidgets.QSpacerItem(
                40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
            horizontalLayout.addItem(spacer)
            self.verticalLayout.addLayout(horizontalLayout)

        spacer = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacer)

        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        Download = QtWidgets.QPushButton()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(
            ":/img/download.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Download.setIcon(icon)
        Download.setIconSize(QtCore.QSize(50, 50))
        Download.setFont(font)
        Download.setStyleSheet(
            "color: #87CEEB; background: transparent; border: none;")
        Download.setText("Download Selected")
        Download.clicked.connect(lambda: self.downloadSelectedSignal.emit())
        self.verticalLayout.addWidget(
            Download, 0, QtCore.Qt.AlignLeft | QtCore.Qt.AlignBottom)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Results()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())
